;==============================================================================
include \masm32\include\masm32rt.inc
;==============================================================================
.data?

    pMem        dd          ?
    fileLength  dd          ?

    printdlg    PRINTDLG    <?>
    docinfo     DOCINFO     <?>
    choosefont  CHOOSEFONT  <?>
    logfont     LOGFONT     <?>

    rc          RECT        <?>

    fnBuffer    db          256 dup(?)

.code
;==============================================================================
start:
;==============================================================================

    ;-------------------------------------------------
    ; Initialize the PRINTDLG and DOCINFO structures.
    ;-------------------------------------------------

    mov   printdlg.lStructSize, sizeof PRINTDLG
    mov   printdlg.Flags, PD_RETURNDC or PD_USEDEVMODECOPIES
    mov   printdlg.nCopies, 1

    mov   docinfo.cbSize, sizeof DOCINFO
    mov   docinfo.lpszDocName, chr$("Print Test")

    ;---------------------------------------------------
    ; Get a printer device context, or return an error.
    ;---------------------------------------------------

    invoke PrintDlg, addr printdlg

    .if eax == 0

        invoke CommDlgExtendedError

        .if eax
            inkey "Print Dialog returned an error..."
            exit
        .else
            inkey "Print Dialog was canceled or closed..."
            exit
        .endif

    .else

        ;----------------------------------------
        ; Start a print job, or return an error.
        ;----------------------------------------

        invoke StartDoc, printdlg.hDC, addr docinfo

        .if eax == SP_ERROR

            inkey "StartDoc returned an error..."
            invoke DeleteDC, printdlg.hDC
            exit

        .endif

        ;----------------------------------------------------------------
        ; Prepare the printer driver to accept data, or return an error.
        ;----------------------------------------------------------------

        invoke StartPage, printdlg.hDC

        .if eax <= 0

            inkey "StartPage returned an error..."
            invoke EndDoc, printdlg.hDC
            invoke DeleteDC, printdlg.hDC
            exit

        .endif

        ;--------------------------------------
        ; Initialize the CHOOSEFONT structure.
        ;--------------------------------------

        mov   choosefont.lStructSize, sizeof CHOOSEFONT
        push  printdlg.hDC
        pop   choosefont.hDC
        mov   choosefont.lpLogFont, offset logfont
        mov   choosefont.Flags, CF_PRINTERFONTS

        ;----------------------------------
        ; Allow the user to choose a font.
        ;----------------------------------

        invoke ChooseFont, addr choosefont

        ;------------------------------------------------
        ; Create a logical font with the characteristics
        ; that the user selected.
        ;------------------------------------------------

        invoke CreateFontIndirect, choosefont.lpLogFont

        ;--------------------------------------------------
        ; Select the new font into the printer DC and save
        ; save the previously selected font on the stack.
        ;--------------------------------------------------

        invoke SelectObject, printdlg.hDC, eax
        push eax

        ;-----------------------------------------------
        ; Get the first argument from the command line.
        ;-----------------------------------------------

        invoke GetCL, 1, addr fnBuffer

        ;---------------------------------------------------
        ; If the first argument was present then attempt to
        ; print the file on the selected printer.
        ;---------------------------------------------------

        .if eax == 1

            ;----------------------------------------------
            ; Allocate a buffer and read the file into it.
            ;----------------------------------------------

            invoke read_disk_file, addr fnBuffer, addr pMem,
                                   addr fileLength

            ;---------------------------------------------------------
            ; This is supposed to get the width of the printable area
            ; for the selected printer and use it to initialize the
            ; width of the formatting rectangle for DrawTextEx.
            ;---------------------------------------------------------

            invoke GetDeviceCaps, printdlg.hDC, HORZRES
            dec eax
            mov rc.right, eax

            ;---------------------------------------------------------
            ; Include the DT_CALCRECT formatting option so DrawTextEx
            ; will calculate the height of the formatting rectangle.
            ;---------------------------------------------------------

            invoke DrawTextEx, printdlg.hDC, pMem, fileLength, addr rc,
                               DT_CALCRECT or DT_EDITCONTROL or \
                               DT_EXPANDTABS or DT_WORDBREAK,
                               0

            print str$(eax),13,10
            print str$(rc.left),13,10
            print str$(rc.top),13,10
            print str$(rc.right),13,10
            print str$(rc.bottom),13,10

            ;----------------
            ; Draw the text.
            ;----------------

            invoke DrawTextEx, printdlg.hDC, pMem, fileLength, addr rc,
                               DT_EDITCONTROL or \
                               DT_EXPANDTABS or DT_WORDBREAK,
                               0

            print str$(eax),13,10

            ;----------------------------
            ; Free the allocated buffer.
            ;----------------------------

            invoke GlobalFree, pMem

            ;-----------------------------------------------
            ; Recover the previously selected font from the
            ; stack and select it back into the printer DC.
            ;-----------------------------------------------

            pop eax
            invoke SelectObject, printdlg.hDC, eax

        .endif

        ;------------------------------------------------------
        ; Notify the printer driver that the page is complete.
        ;------------------------------------------------------

        invoke EndPage, printdlg.hDC

        .if eax <= 0

            inkey  "EndPage returned an error..."
            invoke EndDoc, printdlg.hDC
            invoke DeleteDC, printdlg.hDC
            exit

        .endif

        ;--------------------
        ; End the print job.
        ;--------------------

        invoke EndDoc, printdlg.hDC

        .if eax <= 0

            inkey  "EndDoc returned an error..."
            invoke DeleteDC, printdlg.hDC
            exit

        .endif

        invoke DeleteDC, printdlg.hDC

    .endif

    inkey "Press any key to exit..."

    exit
;==============================================================================
end start
